@@ -59,6 +59,8 @@ enum {
   PROP_0,
   PROP_PAINTABLE,
   PROP_GL_CONTEXT,
+  PROP_USES_GL,
+  PROP_DISPLAY,
 
   N_PROPS,
 };
@@ -68,20 +70,25 @@ GST_DEBUG_CATEGORY (gtk_debug_gst_sink);
 
 #define FORMATS "{ BGRA, ARGB, RGBA, ABGR, RGB, BGR }"
 
-#define NOGL_CAPS GST_VIDEO_CAPS_MAKE (FORMATS)
+#define MEMORY_TEXTURE_CAPS GST_VIDEO_CAPS_MAKE (FORMATS)
+
+#define GL_TEXTURE_CAPS \
+                     "video/x-raw(" GST_CAPS_FEATURE_MEMORY_GL_MEMORY "), " \
+                     "format = (string) RGBA, " \
+                     "width = " GST_VIDEO_SIZE_RANGE ", " \
+                     "height = " GST_VIDEO_SIZE_RANGE ", " \
+                     "framerate = " GST_VIDEO_FPS_RANGE ", " \
+                     "texture-target = (string) 2D"
+
+#define DMABUF_TEXTURE_CAPS GST_VIDEO_DMA_DRM_CAPS_MAKE
 
 static GstStaticPadTemplate gtk_gst_sink_template =
 GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
-    GST_STATIC_CAPS (GST_VIDEO_DMA_DRM_CAPS_MAKE "; "
-                     "video/x-raw(" GST_CAPS_FEATURE_MEMORY_GL_MEMORY "), "
-                     "format = (string) RGBA, "
-                     "width = " GST_VIDEO_SIZE_RANGE ", "
-                     "height = " GST_VIDEO_SIZE_RANGE ", "
-                     "framerate = " GST_VIDEO_FPS_RANGE ", "
-                     "texture-target = (string) 2D"
-                     "; " NOGL_CAPS)
+    GST_STATIC_CAPS(DMABUF_TEXTURE_CAPS "; "
+                    GL_TEXTURE_CAPS "; "
+                    MEMORY_TEXTURE_CAPS)
     );
 
 #undef GST_VIDEO_DMA_DRM_CAPS_MAKE_STR
@@ -204,34 +211,43 @@ gtk_gst_sink_get_caps (GstBaseSink *bsink,
                        GstCaps     *filter)
 {
   GtkGstSink *self = GTK_GST_SINK (bsink);
-  GstCaps *tmp;
-  GstCaps *result;
+  GstCaps *unfiltered, *tmp, *result;
 
-  if (self->gst_context)
-    {
-      GdkDisplay *display = gdk_gl_context_get_display (self->gdk_context);
-      GdkDmabufFormats *formats = gdk_display_get_dmabuf_formats (display);
+  unfiltered = gst_caps_new_empty ();
 
-      tmp = gst_pad_get_pad_template_caps (GST_BASE_SINK_PAD (bsink));
-      tmp = gst_caps_make_writable (tmp);
-      add_drm_formats_and_modifiers (tmp, formats);
-    }
-  else
+  if (self->gdk_display)
     {
-      tmp = gst_caps_from_string (NOGL_CAPS);
+      GdkDmabufFormats *formats = gdk_display_get_dmabuf_formats (self->gdk_display);
+
+      if (formats && gdk_dmabuf_formats_get_n_formats (formats) > 0)
+        {
+          tmp = gst_caps_from_string (DMABUF_TEXTURE_CAPS);
+          add_drm_formats_and_modifiers (tmp, formats);
+          gst_caps_append (unfiltered, tmp);
+        }
     }
-  GST_DEBUG_OBJECT (self, "advertising own caps %" GST_PTR_FORMAT, tmp);
+
+  if (self->gdk_context)
+    {
+      tmp = gst_caps_from_string (GL_TEXTURE_CAPS);
+      gst_caps_append (unfiltered, tmp);
+    }
+
+  tmp = gst_caps_from_string (MEMORY_TEXTURE_CAPS);
+  gst_caps_append (unfiltered, tmp);
+
+  GST_DEBUG_OBJECT (self, "advertising own caps %" GST_PTR_FORMAT, unfiltered);
 
   if (filter)
     {
       GST_DEBUG_OBJECT (self, "intersecting with filter caps %" GST_PTR_FORMAT, filter);
 
-      result = gst_caps_intersect_full (filter, tmp, GST_CAPS_INTERSECT_FIRST);
-      gst_caps_unref (tmp);
+      result = gst_caps_intersect_full (filter, unfiltered, GST_CAPS_INTERSECT_FIRST);
+      gst_caps_unref (unfiltered);
     }
   else
     {
-      result = tmp;
+      result = unfiltered;
     }
 
   GST_DEBUG_OBJECT (self, "returning caps: %" GST_PTR_FORMAT, result);
@@ -299,9 +315,6 @@ gtk_gst_sink_propose_allocation (GstBaseSink *bsink,
   gboolean need_pool;
   GstVideoInfo info;
 
-  if (!self->gst_context)
-    return FALSE;
-
   gst_query_parse_allocation (query, &caps, &need_pool);
 
   if (caps == NULL)
@@ -316,47 +329,49 @@ gtk_gst_sink_propose_allocation (GstBaseSink *bsink,
       return TRUE;
     }
 
-  if (!gst_caps_features_contains (gst_caps_get_features (caps, 0), GST_CAPS_FEATURE_MEMORY_GL_MEMORY))
-    return FALSE;
-
-  if (!gst_video_info_from_caps (&info, caps))
+  if (gst_caps_features_contains (gst_caps_get_features (caps, 0), GST_CAPS_FEATURE_MEMORY_GL_MEMORY))
     {
-      GST_DEBUG_OBJECT (self, "invalid caps specified");
-      return FALSE;
-    }
-
-  /* the normal size of a frame */
-  size = info.size;
-
-  if (need_pool)
-    {
-      GST_DEBUG_OBJECT (self, "create new pool");
-      pool = gst_gl_buffer_pool_new (self->gst_context);
-
-      config = gst_buffer_pool_get_config (pool);
-      gst_buffer_pool_config_set_params (config, caps, size, 0, 0);
-      gst_buffer_pool_config_add_option (config, GST_BUFFER_POOL_OPTION_GL_SYNC_META);
-
-      if (!gst_buffer_pool_set_config (pool, config))
+      if (!gst_video_info_from_caps (&info, caps))
         {
-          GST_DEBUG_OBJECT (bsink, "failed setting config");
-          gst_object_unref (pool);
+          GST_DEBUG_OBJECT (self, "invalid caps specified");
           return FALSE;
         }
+
+      /* the normal size of a frame */
+      size = info.size;
+
+      if (need_pool)
+        {
+          GST_DEBUG_OBJECT (self, "create new pool");
+          pool = gst_gl_buffer_pool_new (self->gst_context);
+
+          config = gst_buffer_pool_get_config (pool);
+          gst_buffer_pool_config_set_params (config, caps, size, 0, 0);
+          gst_buffer_pool_config_add_option (config, GST_BUFFER_POOL_OPTION_GL_SYNC_META);
+
+          if (!gst_buffer_pool_set_config (pool, config))
+            {
+              GST_DEBUG_OBJECT (bsink, "failed setting config");
+              gst_object_unref (pool);
+              return FALSE;
+            }
+        }
+
+      /* we need at least 2 buffer because we hold on to the last one */
+      gst_query_add_allocation_pool (query, pool, size, 2, 0);
+      if (pool)
+        gst_object_unref (pool);
+
+      /* we also support various metadata */
+      gst_query_add_allocation_meta (query, GST_VIDEO_META_API_TYPE, 0);
+
+      if (self->gst_context->gl_vtable->FenceSync)
+        gst_query_add_allocation_meta (query, GST_GL_SYNC_META_API_TYPE, 0);
+
+      return TRUE;
     }
 
-  /* we need at least 2 buffer because we hold on to the last one */
-  gst_query_add_allocation_pool (query, pool, size, 2, 0);
-  if (pool)
-    gst_object_unref (pool);
-
-  /* we also support various metadata */
-  gst_query_add_allocation_meta (query, GST_VIDEO_META_API_TYPE, 0);
-
-  if (self->gst_context->gl_vtable->FenceSync)
-    gst_query_add_allocation_meta (query, GST_GL_SYNC_META_API_TYPE, 0);
-
-  return TRUE;
+  return FALSE;
 }
 
 static GdkMemoryFormat
@@ -400,6 +415,7 @@ gtk_gst_sink_texture_from_buffer (GtkGstSink      *self,
                                   graphene_rect_t *viewport)
 {
   GstVideoFrame *frame = g_new (GstVideoFrame, 1);
+  GstMemory *mem;
   GdkTexture *texture;
 
   viewport->origin.x = 0;
@@ -407,7 +423,9 @@ gtk_gst_sink_texture_from_buffer (GtkGstSink      *self,
   viewport->size.width = GST_VIDEO_INFO_WIDTH (&self->v_info);
   viewport->size.height = GST_VIDEO_INFO_HEIGHT (&self->v_info);
 
-  if (gst_is_dmabuf_memory (gst_buffer_peek_memory (buffer, 0)))
+  mem = gst_buffer_peek_memory (buffer, 0);
+
+  if (gst_is_dmabuf_memory (mem))
     {
       GdkDmabufTextureBuilder *builder = NULL;
       const GstVideoMeta *vmeta = gst_buffer_get_video_meta (buffer);
@@ -418,11 +436,10 @@ gtk_gst_sink_texture_from_buffer (GtkGstSink      *self,
       g_clear_pointer (&frame, g_free);
 
       g_return_val_if_fail (vmeta, NULL);
-      g_return_val_if_fail (self->gdk_context, NULL);
       g_return_val_if_fail (self->drm_info.drm_fourcc != DRM_FORMAT_INVALID, NULL);
 
       builder = gdk_dmabuf_texture_builder_new ();
-      gdk_dmabuf_texture_builder_set_display (builder, gdk_gl_context_get_display (self->gdk_context));
+      gdk_dmabuf_texture_builder_set_display (builder, self->gdk_display);
       gdk_dmabuf_texture_builder_set_fourcc (builder, self->drm_info.drm_fourcc);
       gdk_dmabuf_texture_builder_set_modifier (builder, self->drm_info.drm_modifier);
       gdk_dmabuf_texture_builder_set_width (builder, vmeta->width);
@@ -432,7 +449,6 @@ gtk_gst_sink_texture_from_buffer (GtkGstSink      *self,
 
       for (i = 0; i < vmeta->n_planes; i++)
         {
-          GstMemory *mem;
           guint mem_idx, length;
           gsize skip;
 
@@ -460,12 +476,15 @@ gtk_gst_sink_texture_from_buffer (GtkGstSink      *self,
                                                   &error);
       g_object_unref (builder);
       if (!texture)
-        GST_ERROR_OBJECT (self, "Failed to create dmabuf texture: %s", error->message);
+        {
+          GST_ERROR_OBJECT (self, "Failed to create dmabuf texture: %s", error->message);
+          g_error_free (error);
+        }
 
       *pixel_aspect_ratio = ((double) GST_VIDEO_INFO_PAR_N (&self->v_info) /
                              (double) GST_VIDEO_INFO_PAR_D (&self->v_info));
     }
-  else if (self->gdk_context &&
+  else if (gst_is_gl_memory (mem) &&
            gst_video_frame_map (frame, &self->v_info, buffer, GST_MAP_READ | GST_MAP_GL))
     {
       GstGLSyncMeta *sync_meta;
@@ -764,6 +783,11 @@ gtk_gst_sink_set_property (GObject      *object,
       self->gdk_context = g_value_dup_object (value);
       if (self->gdk_context != NULL && !gtk_gst_sink_initialize_gl (self))
         g_clear_object (&self->gdk_context);
+      self->uses_gl = self->gdk_context != NULL;
+      break;
+
+    case PROP_DISPLAY:
+      self->gdk_display = g_value_dup_object (value);
       break;
 
     default:
@@ -785,10 +809,19 @@ gtk_gst_sink_get_property (GObject    *object,
     case PROP_PAINTABLE:
       g_value_set_object (value, self->paintable);
       break;
+
     case PROP_GL_CONTEXT:
       g_value_set_object (value, self->gdk_context);
       break;
 
+    case PROP_DISPLAY:
+      g_value_set_object (value, self->gdk_display);
+      break;
+
+    case PROP_USES_GL:
+      g_value_set_boolean (value, self->uses_gl);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -805,6 +838,7 @@ gtk_gst_sink_dispose (GObject *object)
   g_clear_object (&self->gst_gdk_context);
   g_clear_object (&self->gst_display);
   g_clear_object (&self->gdk_context);
+  g_clear_object (&self->gdk_display);
 
   G_OBJECT_CLASS (gtk_gst_sink_parent_class)->dispose (object);
 }
@@ -849,6 +883,16 @@ gtk_gst_sink_class_init (GtkGstSinkClass * klass)
                          GDK_TYPE_GL_CONTEXT,
                          G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS);
 
+  properties[PROP_DISPLAY] =
+    g_param_spec_object ("display", NULL, NULL,
+                         GDK_TYPE_DISPLAY,
+                         G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS);
+
+  properties[PROP_USES_GL] =
+    g_param_spec_boolean ("uses-gl", NULL, NULL,
+                          TRUE,
+                          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+
   g_object_class_install_properties (gobject_class, N_PROPS, properties);
 
   gst_element_class_set_metadata (gstelement_class,
